@@ -1,24 +1,27 @@
-from opal_common.fetcher.providers.http_fetch_provider import HttpFetcherConfig
-import aiohttp
 import json
-
 from typing import List, Optional
 
-from opal_common.http import is_http_error_response
-from opal_common.schemas.data import DataUpdateReport
-from opal_common.logger import logger
-from opal_client.data.fetcher import DataFetcher
+import aiohttp
 from opal_client.callbacks.register import CallbackConfig, CallbacksRegister
+from opal_client.data.fetcher import DataFetcher
+from opal_common.fetcher.providers.http_fetch_provider import HttpFetcherConfig
+from opal_common.http import is_http_error_response
+from opal_common.logger import logger
+from opal_common.schemas.data import DataUpdateReport
+
 
 class CallbacksReporter:
-    """
-    can send a report to callbacks registered on the callback register
-    """
+    """can send a report to callbacks registered on the callback register."""
+
     def __init__(self, register: CallbacksRegister, data_fetcher: DataFetcher) -> None:
         self._register = register
         self._fetcher = data_fetcher
 
-    async def report_update_results(self, report: DataUpdateReport, extra_callbacks: Optional[List[CallbackConfig]] = None):
+    async def report_update_results(
+        self,
+        report: DataUpdateReport,
+        extra_callbacks: Optional[List[CallbackConfig]] = None,
+    ):
         try:
             # all the urls that will be eventually called by the fetcher
             urls = []
@@ -26,7 +29,9 @@ class CallbacksReporter:
 
             # first we add the callback urls from the callback register
             for entry in self._register.all():
-                config = entry.config or HttpFetcherConfig() # should not be None if we got it from the register
+                config = (
+                    entry.config or HttpFetcherConfig()
+                )  # should not be None if we got it from the register
                 config.data = report_data
                 urls.append((entry.url, config))
 
@@ -41,8 +46,16 @@ class CallbacksReporter:
             # log reports which we failed to send
             for (url, config, result) in report_results:
                 if isinstance(result, Exception):
-                    logger.error("Failed to send report to {url}, info={exc_info}", url=url, exc_info=repr(result))
-                if isinstance(result, aiohttp.ClientResponse) and is_http_error_response(result): # error responses
+                    logger.error(
+                        "Failed to send report to {url}, info={exc_info}",
+                        url=url,
+                        exc_info=repr(result),
+                    )
+                if isinstance(
+                    result, aiohttp.ClientResponse
+                ) and is_http_error_response(
+                    result
+                ):  # error responses
                     try:
                         error_content = await result.json()
                     except json.JSONDecodeError:
@@ -51,7 +64,7 @@ class CallbacksReporter:
                         "Failed to send report to {url}, got response code {status} with error: {error}",
                         url=url,
                         status=result.status,
-                        error=error_content
+                        error=error_content,
                     )
         except:
-            logger.exception("Failed to excute report_update_results")
\ No newline at end of file
+            logger.exception("Failed to excute report_update_results")
