@@ -2,51 +2,56 @@ import asyncio
 import hashlib
 import itertools
 import json
-from typing import List, Optional, Tuple
 import uuid
-import aiohttp
+from typing import List, Optional, Tuple
 
-from aiohttp.client import ClientSession, ClientError
+import aiohttp
+from aiohttp.client import ClientError, ClientSession
 from fastapi_websocket_pubsub import PubSubClient
 from fastapi_websocket_rpc.rpc_channel import RpcChannel
-
-from opal_common.config import opal_common_config
-from opal_common.fetcher.events import FetcherConfig
-from opal_common.schemas.data import (DataEntryReport, DataSourceConfig,
-                                      DataSourceEntry, DataUpdate,
-                                      DataUpdateReport)
-from opal_common.schemas.store import TransactionType
-from opal_common.utils import get_authorization_header
-from opal_common.http import is_http_error_response
-from opal_common.security.sslcontext import get_custom_ssl_context
-from opal_client.callbacks.reporter import CallbacksReporter
 from opal_client.callbacks.register import CallbacksRegister
+from opal_client.callbacks.reporter import CallbacksReporter
 from opal_client.config import opal_client_config
 from opal_client.data.fetcher import DataFetcher
 from opal_client.data.rpc import TenantAwareRpcEventClientMethods
 from opal_client.logger import logger
-from opal_client.policy_store.base_policy_store_client import \
-    BasePolicyStoreClient
-from opal_client.policy_store.policy_store_client_factory import \
-    DEFAULT_POLICY_STORE_GETTER
+from opal_client.policy_store.base_policy_store_client import BasePolicyStoreClient
+from opal_client.policy_store.policy_store_client_factory import (
+    DEFAULT_POLICY_STORE_GETTER,
+)
+from opal_common.config import opal_common_config
+from opal_common.fetcher.events import FetcherConfig
+from opal_common.http import is_http_error_response
+from opal_common.schemas.data import (
+    DataEntryReport,
+    DataSourceConfig,
+    DataSourceEntry,
+    DataUpdate,
+    DataUpdateReport,
+)
+from opal_common.schemas.store import TransactionType
+from opal_common.security.sslcontext import get_custom_ssl_context
+from opal_common.utils import get_authorization_header
 
 
 class DataUpdater:
-    def __init__(self, token: str = None,
-                 pubsub_url: str = None,
-                 data_sources_config_url: str = None,
-                 fetch_on_connect: bool = True,
-                 data_topics: List[str] = None,
-                 policy_store: BasePolicyStoreClient = None,
-                 should_send_reports=None,
-                 data_fetcher: Optional[DataFetcher] = None,
-                 callbacks_register: Optional[CallbacksRegister] = None,
-                 opal_client_id: str = None
-        ):
-        """
-        Keeps policy-stores (e.g. OPA) up to date with relevant data
-        Obtains data configuration on startup from OPAL-server
-        Uses Pub/Sub to subscribe to data update events, and fetches (using FetchingEngine) data from sources.
+    def __init__(
+        self,
+        token: str = None,
+        pubsub_url: str = None,
+        data_sources_config_url: str = None,
+        fetch_on_connect: bool = True,
+        data_topics: List[str] = None,
+        policy_store: BasePolicyStoreClient = None,
+        should_send_reports=None,
+        data_fetcher: Optional[DataFetcher] = None,
+        callbacks_register: Optional[CallbacksRegister] = None,
+        opal_client_id: str = None,
+    ):
+        """Keeps policy-stores (e.g. OPA) up to date with relevant data Obtains
+        data configuration on startup from OPAL-server Uses Pub/Sub to
+        subscribe to data update events, and fetches (using FetchingEngine)
+        data from sources.
 
         Args:
             token (str, optional): Auth token to include in connections to OPAL server. Defaults to CLIENT_TOKEN.
@@ -59,22 +64,44 @@ class DataUpdater:
         # Defaults
         token: str = token or opal_client_config.CLIENT_TOKEN
         pubsub_url: str = pubsub_url or opal_client_config.SERVER_PUBSUB_URL
-        data_sources_config_url: str = data_sources_config_url or opal_client_config.DEFAULT_DATA_SOURCES_CONFIG_URL
+        self._scope_id = opal_client_config.SCOPE_ID
+        self._data_topics = (
+            data_topics if data_topics is not None else opal_client_config.DATA_TOPICS
+        )
+
+        if self._scope_id == "default":
+            data_sources_config_url: str = (
+                data_sources_config_url
+                or opal_client_config.DEFAULT_DATA_SOURCES_CONFIG_URL
+            )
+        else:
+            data_sources_config_url = (
+                f"{opal_client_config.SERVER_URL}/scopes/{self._scope_id}/data"
+            )
+            self._data_topics = [
+                f"{self._scope_id}:data:{topic}" for topic in self._data_topics
+            ]
+
         # Should the client use the default data source to fetch on connect
         self._fetch_on_connect = fetch_on_connect
         # The policy store we'll save data updates into
         self._policy_store = policy_store or DEFAULT_POLICY_STORE_GETTER()
-        # Pub/Sub topics we subscribe to for data updates
-        self._data_topics = data_topics if data_topics is not None else opal_client_config.DATA_TOPICS
-        self._should_send_reports = should_send_reports if should_send_reports is not None else opal_client_config.SHOULD_REPORT_ON_DATA_UPDATES
+
+        self._should_send_reports = (
+            should_send_reports
+            if should_send_reports is not None
+            else opal_client_config.SHOULD_REPORT_ON_DATA_UPDATES
+        )
         # The pub/sub client for data updates
         self._client = None
-        # The task running the Pub/Sub subcribing client
+        # The task running the Pub/Sub subscribing client
         self._subscriber_task = None
         # Data fetcher
         self._data_fetcher = data_fetcher or DataFetcher()
         self._callbacks_register = callbacks_register or CallbacksRegister()
-        self._callbacks_reporter = CallbacksReporter(self._callbacks_register, self._data_fetcher)
+        self._callbacks_reporter = CallbacksReporter(
+            self._callbacks_register, self._data_fetcher
+        )
         self._token = token
         self._server_url = pubsub_url
         self._data_sources_config_url = data_sources_config_url
@@ -86,16 +113,18 @@ class DataUpdater:
         self._stopping = False
         # custom SSL context (for self-signed certificates)
         self._custom_ssl_context = get_custom_ssl_context()
-        self._ssl_context_kwargs = {'ssl': self._custom_ssl_context} if self._custom_ssl_context is not None else {}
+        self._ssl_context_kwargs = (
+            {"ssl": self._custom_ssl_context}
+            if self._custom_ssl_context is not None
+            else {}
+        )
 
     async def __aenter__(self):
         await self.start()
         return self
 
     async def __aexit__(self, exc_type, exc, tb):
-        """
-        Context handler to terminate internal tasks
-        """
+        """Context handler to terminate internal tasks."""
         if not self._stopping:
             await self.stop()
 
@@ -118,8 +147,11 @@ class DataUpdater:
         if update.id is None:
             update.id = uuid.uuid4().hex
         logger.info("Triggering data update with id: {id}", id=update.id)
-        asyncio.create_task(self.update_policy_data(
-            update, policy_store=self._policy_store, data_fetcher=self._data_fetcher))
+        asyncio.create_task(
+            self.update_policy_data(
+                update, policy_store=self._policy_store, data_fetcher=self._data_fetcher
+            )
+        )
 
     async def get_policy_data_config(self, url: str = None) -> DataSourceConfig:
         """
@@ -139,20 +171,26 @@ class DataUpdater:
                     return DataSourceConfig.parse_obj(await response.json())
                 else:
                     error_details = await response.json()
-                    raise ClientError(f"Fetch data sources failed with status code {response.status}, error: {error_details}")
+                    raise ClientError(
+                        f"Fetch data sources failed with status code {response.status}, error: {error_details}"
+                    )
         except:
             logger.exception(f"Failed to load data sources config")
             raise
 
-    async def get_base_policy_data(self, config_url: str = None, data_fetch_reason="Initial load"):
-        """
-        Load data into the policy store according to the data source's config provided in the config URL
+    async def get_base_policy_data(
+        self, config_url: str = None, data_fetch_reason="Initial load"
+    ):
+        """Load data into the policy store according to the data source's
+        config provided in the config URL.
 
         Args:
             config_url (str, optional): URL to retrive data sources config from. Defaults to None ( self._data_sources_config_url).
             data_fetch_reason (str, optional): Reason to log for the update operation. Defaults to "Initial load".
         """
-        logger.info("Performing data configuration, reason: {reason}", reason=data_fetch_reason)
+        logger.info(
+            "Performing data configuration, reason: {reason}", reason=data_fetch_reason
+        )
         sources_config = await self.get_policy_data_config(url=config_url)
         # translate config to a data update
         entries = sources_config.entries
@@ -160,12 +198,13 @@ class DataUpdater:
         self.trigger_data_update(update)
 
     async def on_connect(self, client: PubSubClient, channel: RpcChannel):
-        """
-        Pub/Sub on_connect callback
-        On connection to backend, whether its the first connection,
-        or reconnecting after downtime, refetch the state opa needs.
-        As long as the connection is alive we know we are in sync with the server,
-        when the connection is lost we assume we need to start from scratch.
+        """Pub/Sub on_connect callback On connection to backend, whether its
+        the first connection, or reconnecting after downtime, refetch the state
+        opa needs.
+
+        As long as the connection is alive we know we are in sync with
+        the server, when the connection is lost we assume we need to
+        start from scratch.
         """
         logger.info("Connected to server")
         if self._fetch_on_connect:
@@ -173,8 +212,14 @@ class DataUpdater:
         if opal_common_config.STATISTICS_ENABLED:
             await self._client.wait_until_ready()
             # publish statistics to the server about new connection from client (only if STATISTICS_ENABLED is True, default to False)
-            await self._client.publish([opal_common_config.STATISTICS_ADD_CLIENT_CHANNEL], data={'topics': self._data_topics, 'client_id': self._opal_client_id, 'rpc_id': channel.id})
-
+            await self._client.publish(
+                [opal_common_config.STATISTICS_ADD_CLIENT_CHANNEL],
+                data={
+                    "topics": self._data_topics,
+                    "client_id": self._opal_client_id,
+                    "rpc_id": channel.id,
+                },
+            )
 
     async def on_disconnect(self, channel: RpcChannel):
         logger.info("Disconnected from server")
@@ -186,10 +231,8 @@ class DataUpdater:
             await self._data_fetcher.start()
 
     async def _subscriber(self):
-        """
-        Coroutine meant to be spunoff with create_task to listen in
-        the background for data events and pass them to the data_fetcher
-        """
+        """Coroutine meant to be spunoff with create_task to listen in the
+        background for data events and pass them to the data_fetcher."""
         logger.info("Subscribing to topics: {topics}", topics=self._data_topics)
         self._client = PubSubClient(
             self._data_topics,
@@ -199,7 +242,7 @@ class DataUpdater:
             extra_headers=self._extra_headers,
             keep_alive=opal_client_config.KEEP_ALIVE_INTERVAL,
             server_uri=self._server_url,
-            **self._ssl_context_kwargs
+            **self._ssl_context_kwargs,
         )
         async with self._client:
             await self._client.wait_until_done()
@@ -213,7 +256,9 @@ class DataUpdater:
             try:
                 await asyncio.wait_for(self._client.disconnect(), timeout=3)
             except asyncio.TimeoutError:
-                logger.debug("Timeout waiting for DataUpdater pubsub client to disconnect")
+                logger.debug(
+                    "Timeout waiting for DataUpdater pubsub client to disconnect"
+                )
 
         # stop subscriber task
         if self._subscriber_task is not None:
@@ -222,7 +267,10 @@ class DataUpdater:
             try:
                 await self._subscriber_task
             except asyncio.CancelledError as exc:
-                logger.debug("DataUpdater subscriber task was force-cancelled: {exc}", exc=repr(exc))
+                logger.debug(
+                    "DataUpdater subscriber task was force-cancelled: {exc}",
+                    exc=repr(exc),
+                )
             self._subscriber_task = None
             logger.debug("DataUpdater subscriber task was cancelled")
 
@@ -236,8 +284,9 @@ class DataUpdater:
 
     @staticmethod
     def calc_hash(data):
-        """
-        Calculate an hash (sah256) on the given data, if data isn't a string, it will be converted to JSON.
+        """Calculate an hash (sah256) on the given data, if data isn't a
+        string, it will be converted to JSON.
+
         String are encoded as 'utf-8' prior to hash calculation.
         Returns:
             the hash of the given data (as a a hexdigit string) or '' on failure to process.
@@ -245,15 +294,19 @@ class DataUpdater:
         try:
             if not isinstance(data, str):
                 data = json.dumps(data, default=str)
-            return hashlib.sha256(data.encode('utf-8')).hexdigest()
+            return hashlib.sha256(data.encode("utf-8")).hexdigest()
         except:
             logger.exception("Failed to calculate hash for data {data}", data=data)
             return ""
 
-    async def update_policy_data(self, update: DataUpdate = None, policy_store: BasePolicyStoreClient = None, data_fetcher=None):
-        """
-        fetches policy data (policy configuration) from backend and updates it into policy-store (i.e. OPA)
-        """
+    async def update_policy_data(
+        self,
+        update: DataUpdate = None,
+        policy_store: BasePolicyStoreClient = None,
+        data_fetcher=None,
+    ):
+        """fetches policy data (policy configuration) from backend and updates
+        it into policy-store (i.e. OPA)"""
         policy_store = policy_store or DEFAULT_POLICY_STORE_GETTER()
         if data_fetcher is None:
             data_fetcher = DataFetcher()
@@ -273,18 +326,30 @@ class DataUpdater:
         policy_data_with_urls = await data_fetcher.handle_urls(urls)
         # Save the data from the update
         # We wrap our interaction with the policy store with a transaction
-        async with policy_store.transaction_context(update.id, transaction_type=TransactionType.data) as store_transaction:
+        async with policy_store.transaction_context(
+            update.id, transaction_type=TransactionType.data
+        ) as store_transaction:
             # for intelisense treat store_transaction as a PolicyStoreClient (which it proxies)
             store_transaction: BasePolicyStoreClient
             error_content = None
-            for (url, fetch_config, result), entry in itertools.zip_longest(policy_data_with_urls, entries):
+            for (url, fetch_config, result), entry in itertools.zip_longest(
+                policy_data_with_urls, entries
+            ):
                 fetched_data_successfully = True
 
                 if isinstance(result, Exception):
                     fetched_data_successfully = False
-                    logger.error("Failed to fetch url {url}, got exception: {exc}", url=url, exc=result)
+                    logger.error(
+                        "Failed to fetch url {url}, got exception: {exc}",
+                        url=url,
+                        exc=result,
+                    )
 
-                if isinstance(result, aiohttp.ClientResponse) and is_http_error_response(result): # error responses
+                if isinstance(
+                    result, aiohttp.ClientResponse
+                ) and is_http_error_response(
+                    result
+                ):  # error responses
                     fetched_data_successfully = False
                     try:
                         error_content = await result.json()
@@ -292,7 +357,7 @@ class DataUpdater:
                             "Failed to fetch url {url}, got response code {status} with error: {error}",
                             url=url,
                             status=result.status,
-                            error=error_content
+                            error=error_content,
                         )
                     except json.JSONDecodeError:
                         error_content = await result.text()
@@ -300,9 +365,11 @@ class DataUpdater:
                             "Failed to decode response from url:{url}, got response code {status} with response: {error}",
                             url=url,
                             status=result.status,
-                            error=error_content
+                            error=error_content,
                         )
-                store_transaction._update_remote_status(url=url, status=fetched_data_successfully, error=str(error_content))
+                store_transaction._update_remote_status(
+                    url=url, status=fetched_data_successfully, error=str(error_content)
+                )
 
                 if fetched_data_successfully:
                     # get path to store the URL data (default mode (None) is as "" - i.e. as all the data at root)
@@ -311,18 +378,24 @@ class DataUpdater:
                     if policy_store_path is None:
                         policy_store_path = ""
                     # fix opa_path (if not empty must start with "/" to be nested under data)
-                    if policy_store_path != "" and not policy_store_path.startswith("/"):
+                    if policy_store_path != "" and not policy_store_path.startswith(
+                        "/"
+                    ):
                         policy_store_path = f"/{policy_store_path}"
                     policy_data = result
                     # Create a report on the data-fetching
-                    report = DataEntryReport(entry=entry, hash=self.calc_hash(policy_data), fetched=True)
+                    report = DataEntryReport(
+                        entry=entry, hash=self.calc_hash(policy_data), fetched=True
+                    )
                     logger.info(
                         "Saving fetched data to policy-store: source url='{url}', destination path='{path}'",
                         url=url,
-                        path=policy_store_path or '/'
+                        path=policy_store_path or "/",
                     )
                     try:
-                        await store_transaction.set_policy_data(policy_data, path=policy_store_path)
+                        await store_transaction.set_policy_data(
+                            policy_data, path=policy_store_path
+                        )
                         # No exception we we're able to save to the policy-store
                         report.saved = True
                         # save the report for the entry
@@ -343,5 +416,11 @@ class DataUpdater:
         if self._should_send_reports:
             # spin off reporting (no need to wait on it)
             whole_report = DataUpdateReport(update_id=update.id, reports=reports)
-            extra_callbacks = self._callbacks_register.normalize_callbacks(update.callback.callbacks)
-            asyncio.create_task(self._callbacks_reporter.report_update_results(whole_report, extra_callbacks))
+            extra_callbacks = self._callbacks_register.normalize_callbacks(
+                update.callback.callbacks
+            )
+            asyncio.create_task(
+                self._callbacks_reporter.report_update_results(
+                    whole_report, extra_callbacks
+                )
+            )
