@@ -1,5 +1,5 @@
-from .opal_client import OpalClient
+from .client import OpalClient
 
 client = OpalClient()
 # expose app for Uvicorn
-app = client.app
\ No newline at end of file
+app = client.app
